module Git
  # Git::Log parses output passed from Git#log and creates a hash with author as a key and email as value
  class Log
    # We are expecting output from "git log" command
    def initialize(log)
      @log = log
    end

    # Iterate through each line in the log
    # filter out whitespace characters, split the line by semicolon
    # assign the returned pair (i.e. author and email) to author and email variables
    # then create hash with author as key and email as value and return it.
    # Return from function if we have 5 authors
    def author_email
      author_email = Hash.new
      @log.each_line do |line|
        author, mail = line.strip.split(';')

        author_email[author] = mail unless author_email.key?(author)

        return author_email if author_email.size == 5
      end
      author_email
    end

    # To ensure log will be text when we want
    def to_s
      @log
    end
  end

  # Take a git command and execute
  # Abort if command didn't execute successfully
  # then return output of the command.
  # Print custom message if required.
  def self.execute(command, msg = '')
    output = `git #{command}`

    abort "Git command failed: '#{command}'" + " #{msg}" unless $?.success?

    return output
  end

  # Check if file is tracked with git.
  # If it is not, exit status 1 will be returned by git.
  # so it will be aborted by execute method.
  #
  # Create instance of Git::Log and pass it output from git log command
  # return 5 last authors of a particular file
  # %an returns author, semicolon for effortless parsing, %ae return email of author
  def self.log(file)
    Git.execute("ls-files --error-unmatch #{file}", "File is not added in git: #{file}")
    Log.new(execute("log --pretty=format:'%an;%ae' #{file}"))
  end
end
